/*
 * (c) Copyright 2003 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */

package edu.virginia.cs.escjava.properties;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbenchPropertyPage;

import edu.virginia.cs.escjava.EscjavaNature;
import edu.virginia.cs.escjava.EscjavaPlugin;
import edu.virginia.cs.escjava.EscjavaPreferenceStore;
import edu.virginia.cs.escjava.EscjavaConfiguration;
/**
 * property page for project-specific checklipse properties
 * @author Marco van Meegen
 */
public class EscjavaPropertyPage extends FieldEditorPreferencePage implements IWorkbenchPropertyPage {

	/** remember enabled state */
	private boolean lastEnabledState = false;

	/**
	 * constructor
	 */
	public EscjavaPropertyPage() {
		super(GRID);
		// use dummy preference store to gather values
		setPreferenceStore(new EscjavaPreferenceStore());
		setDescription("ESC/Java Preferences");
	}

	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */

	public void createFieldEditors() {
		addField(new BooleanFieldEditor(EscjavaConfiguration.P_ENABLED, "Enable ESC/Java", getFieldEditorParent()));
	}

	/** 
	 * invalidate checker cache if properties change
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		EscjavaConfiguration.invalidate();
	}

	/** 
	 * check if configuration is valid by creating a checker which fits
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		// first save preferences
		boolean ok = false;
		try {
			ok = super.performOk();
		} catch (Throwable ex) {
			ex.printStackTrace();
		}
		
		if (ok) {
			// now try to create a checker
			try {
				// remove cached checker
				EscjavaConfiguration.invalidate(getProject());
				updateConfiguration();
				EscjavaConfiguration.getChecker(getProject());
			} catch (Throwable ex) {
				// something went wrong, stay here
				setErrorMessage("Invalid Configuration: " + ex.getMessage());
				return false;
			}
		}

		// now activate/deactivate checkstyle
		// and perform build/remove markers
		updateCheckstyle();

		return ok;
	}

	/**
	 * sets the persistent property
	 * @param property property name to set
	 * @param value value
	 * @return true, if property was set; false if failed
	 */
	private boolean setPersistentProperty(String property, String value) {
		getPreferenceStore().setValue(property, value);
		return true;
	}

	/**
	 * gets the persistent property
	 * @param property property name
	 * @return value value
	 */
	private String getPersistentProperty(String property) {
		return getPreferenceStore().getString(property);
	}
	/**
	 * The element.
	 */
	private IAdaptable element;

	/** @return the element associated with this property page
	 */
	public IAdaptable getElement() {
		return element;
	}

	/** @return project for this page */
	private IProject getProject() {
		return (IProject) getElement().getAdapter(IProject.class);
	}

	/**
	 * Sets the element that owns properties shown on this page.
	 *
	 * @param element the element
	 */
	public void setElement(IAdaptable element) {
		this.element = element;
		EscjavaPreferenceStore prefs = (EscjavaPreferenceStore) getPreferenceStore();
		prefs.setProject((IProject) element.getAdapter(IProject.class));
		try {
			prefs.load();
		} catch (IOException ex) {
			EscjavaPlugin.log("project properties: load failed", ex);
		}
	}

	/**
	 * moves preference values from resource store to internal dummy store
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#initialize()
	 */
	protected void initialize() {
		super.initialize();
		getPreferenceStore().setValue(
			EscjavaConfiguration.P_CONFIG_FILE,
			getPersistentProperty(EscjavaConfiguration.P_CONFIG_FILE));

		// set enabled property
		lastEnabledState = Boolean.valueOf(getPersistentProperty(EscjavaConfiguration.P_ENABLED)).booleanValue();
		getPreferenceStore().setValue(EscjavaConfiguration.P_ENABLED, lastEnabledState);

		// now init fields from preferences
		super.initialize();
	}

	/** updates the configuration info after properties have been changed 
	 *  @throws IOException if save fails
	 */
	protected void updateConfiguration() throws IOException {
		setPersistentProperty(
			EscjavaConfiguration.P_CONFIG_FILE,
			getPreferenceStore().getString(EscjavaConfiguration.P_CONFIG_FILE));

		EscjavaPreferenceStore store = (EscjavaPreferenceStore) getPreferenceStore();
		store.save();
	}

	/** updates the checkstyle state according to project settings.
	 * The property will be persisted and a builder will be added/removed
	 * accordingly.
	 */
	protected void updateCheckstyle() {
		IProject project = getProject();

		// remove cached checker
		EscjavaConfiguration.invalidate(project);

		// store preference
		boolean enabled = getPreferenceStore().getBoolean(EscjavaConfiguration.P_ENABLED);

		if (enabled != lastEnabledState) {
			try {
				if (enabled) {
					// add nature
					boolean added = EscjavaNature.add(project);
					if (added) {
						rebuildProject(project);
					}
				} else {
					// remove nature and all markers
					EscjavaNature.remove(project);
					project.deleteMarkers(EscjavaPlugin.CHECKCLIPSE_MARKER_ID, true, IProject.DEPTH_INFINITE);
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		} else {
			// rebuild always on ok
			rebuildProject(project);
		}
	}

	/** rebuild the project 
	 * @param project project to rebuild
	 */
	private void rebuildProject(final IProject project) {
		boolean autoRebuild = EscjavaPlugin.getDefault().getPreferenceStore().getBoolean(EscjavaConfiguration.P_AUTO_REBUILD);
		if (autoRebuild) {
			// TODO proper error handling
			// rebuild project
			ProgressMonitorDialog dlg = new ProgressMonitorDialog(getShell());
	
			try {
				dlg.run(true, true, new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						try {
							project.build(
								IncrementalProjectBuilder.FULL_BUILD,
								EscjavaPlugin.CHECKCLIPSE_BUILDER_ID,
								null,
								monitor);
						} catch (CoreException ex) {
							ex.printStackTrace();
						}
					}
				});
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				dlg.close();
			}
		}
	}

}
