/*
 * (c) Copyright 2002 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */
package edu.virginia.cs.escjava.properties;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;

/**
 * File Field Editor for Files inside Workspace.
 * File paths are transformed to internal relative paths.
 * @author Marco van Meegen
 */
public class ResourceFileFieldEditor extends FileFieldEditor {

	/**
	 * List of legal file extension suffixes, or <code>null</code>
	 * for system defaults.
	 */
	private String[] extensions = null;

	/** project the resource file should come from */
	private IProject project;

	/**
	 * 
	 */
	public ResourceFileFieldEditor() {
		super();
	}

	/**
	 * @param name field name
	 * @param labelText text for label
	 * @param parent parent
	 * @param project project
	 */
	public ResourceFileFieldEditor(String name, String labelText, Composite parent, IProject project) {
		super(name, labelText, parent);
		this.project = project;
		// TODO needs to be fixed in FileFieldEditor, here to late
		setValidateStrategy(VALIDATE_ON_KEY_STROKE);
	}

	/** 
	 * Opens the file chooser dialog and returns the selected file.
	 * @return changed path
	 */
	protected String changePressed() {
		// path is workspace-relative (/project/pathtoresource)
		String relativePath = getTextControl().getText();
		String absolutePath = relativePath;
		try {
			IFile internalFile = project.getWorkspace().getRoot().getFile(new Path(relativePath));
			absolutePath = internalFile.getLocation().toOSString();
		} catch (Exception ex) {
			absolutePath = relativePath;
		}

		File f = new File(absolutePath);
		if (!f.exists()) {
			f = null;
		}
		File d = getFile(f);
		if (d == null) {
			return null;
		}
		
		// convert path to relative if possible
		IPath path = new Path(d.getAbsolutePath());
		IFile file = project.getWorkspace().getRoot().getFileForLocation(path);
		if (file != null) {
			return file.getFullPath().toString();
		}
		return path.toString();
	}

	/**
	 * Helper to open the file chooser dialog.
	 * @param startingDirectory directory to start with
	 * @return chosen File
	 */
	private File getFile(File startingDirectory) {

		FileDialog dialog = new FileDialog(getShell(), SWT.OPEN);
		if (startingDirectory != null) {
			dialog.setFileName(startingDirectory.getPath());
		}
		if (extensions != null) {
			dialog.setFilterExtensions(extensions);
		}

		String filterPath = project.getLocation().toOSString();
		dialog.setFilterPath(filterPath);
		String file = dialog.open();
		if (file != null) {
			file = file.trim();
			if (file.length() > 0) {
				return new File(file);
			}
		}

		return null;
	}

	/** Method declared on StringFieldEditor.
	 * Checks whether the text input field specifies an existing file.
	 * @return true if ok
	 */
	protected boolean checkState() {

		String msg = null;

		String relativePath = getTextControl().getText();
		String absolutePath = relativePath;
		try {
			IFile internalFile = project.getWorkspace().getRoot().getFile(new Path(relativePath));
			absolutePath = internalFile.getLocation().toOSString();
		} catch (Exception ex) {
			absolutePath = relativePath;
		}
		String path = absolutePath;
		
		if (path != null) {
			path = path.trim();
		} else {
			path = ""; //$NON-NLS-1$
		}
		if (path.length() == 0) {
			if (!isEmptyStringAllowed()) {
				msg = getErrorMessage();
			}
		} else {
			File file = new File(path);
			if (!file.isFile()) {
				msg = getErrorMessage();
			}
		}

		if (msg != null) { // error
			showErrorMessage(msg);
			return false;
		}

		// OK!
		clearErrorMessage();
		return true;
	}
	/**
	 * Sets this file field editor's file extension filter.
	 *
	 * @param extensions a list of file extension, or <code>null</code> 
	 * to set the filter to the system's default value
	 */
	public void setFileExtensions(String[] extensions) {
		this.extensions = extensions;
	}
}
